@@ -28,11 +28,8 @@ def main() -> None:
     if dump_file:
         dump_request(dump_file, request)
 
-    # Create response
-    response = CodeGeneratorResponse()
-
     # Generate code
-    generate_code(request, response)
+    response = generate_code(request)
 
     # Serialise response message
     output = response.SerializeToString()
