@@ -30,6 +30,7 @@ reference to `A` to `B`'s `fields` attribute.
 """
 
 
+import builtins
 import betterproto
 from betterproto import which_one_of
 from betterproto.casing import sanitize_name
@@ -58,8 +59,7 @@ from betterproto.lib.google.protobuf.compiler import CodeGeneratorRequest
 import re
 import textwrap
 from dataclasses import dataclass, field
-from typing import Dict, Iterable, Iterator, List, Optional, Set, Text, Type, Union
-import sys
+from typing import Dict, Iterable, Iterator, List, Optional, Set, Type, Union
 
 from ..casing import sanitize_name
 from ..compile.importing import get_type_reference, parse_source_type_name
@@ -237,6 +237,7 @@ class OutputTemplate:
     imports: Set[str] = field(default_factory=set)
     datetime_imports: Set[str] = field(default_factory=set)
     typing_imports: Set[str] = field(default_factory=set)
+    builtins_import: bool = False
     messages: List["MessageCompiler"] = field(default_factory=list)
     enums: List["EnumDefinitionCompiler"] = field(default_factory=list)
     services: List["ServiceCompiler"] = field(default_factory=list)
@@ -268,6 +269,8 @@ class OutputTemplate:
         imports = set()
         if any(x for x in self.messages if any(x.deprecated_fields)):
             imports.add("warnings")
+        if self.builtins_import:
+            imports.add("builtins")
         return imports
 
 
@@ -283,6 +286,7 @@ class MessageCompiler(ProtoContentBase):
         default_factory=list
     )
     deprecated: bool = field(default=False, init=False)
+    builtins_types: Set[str] = field(default_factory=set)
 
     def __post_init__(self) -> None:
         # Add message to output file
@@ -379,6 +383,8 @@ class FieldCompiler(MessageCompiler):
         betterproto_field_type = (
             f"betterproto.{self.field_type}_field({self.proto_obj.number}{field_args})"
         )
+        if self.py_name in dir(builtins):
+            self.parent.builtins_types.add(self.py_name)
         return f"{name}{annotations} = {betterproto_field_type}"
 
     @property
@@ -386,6 +392,8 @@ class FieldCompiler(MessageCompiler):
         args = []
         if self.field_wraps:
             args.append(f"wraps={self.field_wraps}")
+        if self.optional:
+            args.append(f"optional=True")
         return args
 
     @property
@@ -411,9 +419,16 @@ class FieldCompiler(MessageCompiler):
             imports.add("Dict")
         return imports
 
+    @property
+    def use_builtins(self) -> bool:
+        return self.py_type in self.parent.builtins_types or (
+            self.py_type == self.py_name and self.py_name in dir(builtins)
+        )
+
     def add_imports_to(self, output_file: OutputTemplate) -> None:
         output_file.datetime_imports.update(self.datetime_imports)
         output_file.typing_imports.update(self.typing_imports)
+        output_file.builtins_import = output_file.builtins_import or self.use_builtins
 
     @property
     def field_wraps(self) -> Optional[str]:
@@ -434,6 +449,10 @@ class FieldCompiler(MessageCompiler):
             and not is_map(self.proto_obj, self.parent)
         )
 
+    @property
+    def optional(self) -> bool:
+        return self.proto_obj.proto3_optional
+
     @property
     def mutable(self) -> bool:
         """True if the field is a mutable type, otherwise False."""
@@ -449,10 +468,12 @@ class FieldCompiler(MessageCompiler):
         )
 
     @property
-    def default_value_string(self) -> Union[Text, None, float, int]:
+    def default_value_string(self) -> str:
         """Python representation of the default proto value."""
         if self.repeated:
             return "[]"
+        if self.optional:
+            return "None"
         if self.py_type == "int":
             return "0"
         if self.py_type == "float":
@@ -463,6 +484,14 @@ class FieldCompiler(MessageCompiler):
             return '""'
         elif self.py_type == "bytes":
             return 'b""'
+        elif self.field_type == "enum":
+            enum_proto_obj_name = self.proto_obj.type_name.split(".").pop()
+            enum = next(
+                e
+                for e in self.output_file.enums
+                if e.proto_obj.name == enum_proto_obj_name
+            )
+            return enum.default_value_string
         else:
             # Message type
             return "None"
@@ -507,9 +536,14 @@ class FieldCompiler(MessageCompiler):
 
     @property
     def annotation(self) -> str:
+        py_type = self.py_type
+        if self.use_builtins:
+            py_type = f"builtins.{py_type}"
         if self.repeated:
-            return f"List[{self.py_type}]"
-        return self.py_type
+            return f"List[{py_type}]"
+        if self.optional:
+            return f"Optional[{py_type}]"
+        return py_type
 
 
 @dataclass
@@ -656,7 +690,9 @@ class ServiceMethodCompiler(ProtoContentBase):
             self.output_file.typing_imports.add("AsyncIterable")
             self.output_file.typing_imports.add("Iterable")
             self.output_file.typing_imports.add("Union")
-        if self.server_streaming:
+
+        # Required by both client and server
+        if self.client_streaming or self.server_streaming:
             self.output_file.typing_imports.add("AsyncIterator")
 
         super().__post_init__()  # check for unset fields
